package net.sf.itcb.extension.mojo;

import org.apache.maven.plugin.AbstractMojo;
import org.apache.maven.plugin.MojoExecutionException;
import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

/**
 * Make a catalog file according to XSD files in all jar files
 *
 * @goal makeCatalog
 * 
 * @phase package
 */
public class CatalogMaker
    extends AbstractMojo
{
    /**
     * Location of the file.
     * @parameter expression="${project.build.directory}/CatalogMaker/talendDependencies.xml"
     * @required
     */
    private File outputFile;
    
    /**
     * Location of jar directory to list.
     * @parameter expression="${project.build.directory}/xsdImported/WEB-INF/"
     * @required
     */
    private File schemasDirectory;  
    
    public void execute()
        throws MojoExecutionException
    {
        File f = outputFile;
        
        if ( !f.getAbsoluteFile().exists() )
        {
            f.getParentFile().mkdirs();
        }

        FileWriter w = null;
        try
        {
            w = new FileWriter( f );
            
			addCatalogEntry(w, schemasDirectory, true);
			
            addCatalogEntry(w, new File(schemasDirectory.getAbsolutePath()+ "\\" + "schemas"), false);
        }
        catch ( IOException e )
        {
            throw new MojoExecutionException( "IO Exception " , e );
	    } catch (ParserConfigurationException e) {
	    	throw new MojoExecutionException( "ParserConfigurationException " , e );
		} catch (SAXException e) {
			throw new MojoExecutionException( "SAXException " , e );
		}
        finally
        {
            if ( w != null )
            {
                try
                {
                    w.close();
                }
                catch ( IOException e )
                {
                    // ignore
                }
            }
        }
    }


	private void addCatalogEntry(FileWriter w, File schemaDirectoryParam, boolean isProjectSchemaDirectory) throws IOException, ParserConfigurationException, SAXException, MojoExecutionException {
		String[] schemaFiles = schemaDirectoryParam.list();
		if(schemaFiles != null) {
			for (int i = 0; i < schemaFiles.length; i++) {
				if( schemaFiles[i].endsWith(".xsd")  ) {
					// PUBLIC "http://www.mediacourtage.fr/namespace/" "schemas/mediacourtage-common.xsd"
						w.write( "PUBLIC \"" );
						
						
						w.write( resolveNamespace(schemaDirectoryParam, schemaFiles[i]) );
						w.write( "\" \"" );
						if(!isProjectSchemaDirectory) {
							w.write("schemas/");
						}
						w.write(schemaFiles[i] );
						w.write( "\"\n" );
				}
			}
		}
	}


	private String resolveNamespace(File schemaDirectory, String file) throws ParserConfigurationException, SAXException, IOException, MojoExecutionException {
		getLog().debug("Analysing XSD file : " +  schemaDirectory.getAbsolutePath() + "/" + file);
		File xsdFile = new File(schemaDirectory.getAbsolutePath() + "/" + file);
		
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		DocumentBuilder db = dbf.newDocumentBuilder();
		Document doc = db.parse(xsdFile);
		NodeList nodeList = doc.getElementsByTagName("xsd:schema");
		getLog().debug("schema : " + String.valueOf(doc.getElementsByTagName("schema").getLength()));

		if(nodeList.getLength() < 1) {
			throw new MojoExecutionException("A root element 'schema' must be defined");
		}
		NamedNodeMap nodeSchemaAttributes = nodeList.item(0).getAttributes();
		if(nodeSchemaAttributes.getLength() < 1) {
			throw new MojoExecutionException("An attribute targetNamespace must be defined in 'schema' element");
		}
		Node targetNamespace = nodeSchemaAttributes.getNamedItem("targetNamespace");
		getLog().debug("targetNamespace.getNodeValue() : " + targetNamespace.getNodeValue());
		return targetNamespace.getNodeValue();
	}

}
